/*	$Id: ueaglereg.h,v 1.10 2004/08/25 16:26:47 damien Exp $	*/

/*-
 * Copyright (c) 2003, 2004
 *	Damien Bergamini <damien.bergamini@free.fr>. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice unmodified, this list of conditions, and the following
 *    disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/* Analog Devices vendor and product ids */
#define USB_VENDOR_ADI			0x1110
#define USB_PRODUCT_ADI_EAGLEI		0x900f
#define USB_PRODUCT_ADI_EAGLEII		0x9023
#define USB_PRODUCT_ADI_EAGLEIIC	0x9021

/* U.S. Robotics vendor and product ids */
#define USB_VENDOR_USR			0x0baf
#define USB_PRODUCT_USR_MILLER_A	0x00f1
#define USB_PRODUCT_USR_MILLER_B	0x00f9
#define USB_PRODUCT_USR_HEINEKEN_A	0x00f5
#define USB_PRODUCT_USR_HEINEKEN_B	0x00f7

/* OPTN: default values from analog devices */
#ifndef UEAGLE_OPTN0
#define UEAGLE_OPTN0	0x80020066
#endif
#ifndef UEAGLE_OPTN2
#define UEAGLE_OPTN2	0x23700000
#endif
#ifndef UEAGLE_OPTN7
#define UEAGLE_OPTN7	0x02cd8044
#endif

#define UEAGLE_CONFIG_NO	1

#define UEAGLE_ESISTRING	4

#define UEAGLE_INTR_IFACE_NO	0
#define	UEAGLE_US_IFACE_NO	1
#define UEAGLE_DS_IFACE_NO	2

#define UEAGLE_TX_PIPE		0x02
#define UEAGLE_IDMA_PIPE	0x04
#define UEAGLE_INTR_PIPE	0x84
#define UEAGLE_RX_PIPE		0x88

#define UEAGLE_SETBLOCK    0x0001
#define UEAGLE_SETMODE     0x0003
#define UEAGLE_SET2183DATA 0x0004

#define UEAGLE_LOOPBACKOFF 0x0002
#define UEAGLE_LOOPBACKON  0x0003
#define UEAGLE_BOOTIDMA    0x0006
#define UEAGLE_STARTRESET  0x0007
#define UEAGLE_ENDRESET    0x0008
#define UEAGLE_SWAPMAILBOX (0x3fcd | 0x4000)
#define UEAGLE_MPTXSTART   (0x3fce | 0x4000)
#define UEAGLE_MPTXMAILBOX (0x3fd6 | 0x4000)
#define UEAGLE_MPRXMAILBOX (0x3fdf | 0x4000)

/* structure describing a block within a fw page */
typedef struct {
	uWord	wHdr;
#define UEAGLE_BIHDR	0xabcd
	uWord	wAddress;
	uWord	wSize;
	uWord	wOvlOffset;
	uWord	wOvl;	/* overlay */
	uWord	wLast;
} UPACKED block_info_t;
#define BLOCK_INFO_SIZE 12

/* structure representing a CMV (Configuration and Management Variable) */
typedef struct {
	uWord	wPreamble;
#define PREAMBLE 0x535c
	uByte	bDirection;
#define MODEMTOHOST 0x01
#define HOSTTOMODEM 0x10
	uByte	bFunction;
#define FUNCTION_TYPE(f)    ((f) >> 4)
#define MEMACCESS	0x1
#define ADSLDIRECTIVE	0x7

#define FUNCTION_SUBTYPE(f) ((f) & 0x0f)
#define REQUESTREAD	0x0
#define REQUESTWRITE	0x1
#define REPLYREAD	0x2
#define REPLYWRITE	0x3
#define KERNELREADY	0x0
#define MODEMREADY	0x1

#define MAKEFUNCTION(t, s) (((t) & 0xf) << 4 | ((s) & 0xf))
	uWord	wIndex;
	uDWord	dwSymbolicAddress;
#define MAKESA(a, b, c, d)						\
	(((c) & 0xff) << 24 |						\
	 ((d) & 0xff) << 16 |						\
	 ((a) & 0xff) << 8  |						\
	 ((b) & 0xff))

#define SA_CNTL MAKESA('C', 'N', 'T', 'L')
#define SA_DIAG MAKESA('D', 'I', 'A', 'G')
#define SA_INFO MAKESA('I', 'N', 'F', 'O')
#define SA_OPTN MAKESA('O', 'P', 'T', 'N')
#define SA_RATE MAKESA('R', 'A', 'T', 'E')
#define SA_STAT MAKESA('S', 'T', 'A', 'T')
	uWord	wOffsetAddress;
	uDWord	dwData;
} UPACKED cmv_t;
#define CMV_SIZE 16

/* structure representing swap information */
typedef struct {
	uByte	bSwapPageNo;
	uByte	bOvl;	/* overlay */
} UPACKED swap_info_t;

/* structure representing interrupt data */
typedef struct {
	uByte  	bType;
	uByte	bNotification;
	uWord	wValue;
	uWord	wIndex;
	uWord	wLength;
	uWord	wInterrupt;
#define INT_LOADSWAPPAGE 0x0001
#define INT_INCOMINGCMV  0x0002
	union {
		struct {
			swap_info_t	swapinfo;
			uWord		wDataSize;
		} UPACKED s1;

		struct {
			cmv_t	cmv;
			uWord	wDataSize;
		} UPACKED s2;
	} UPACKED u;
#define bSwapPageNo	u.s1.swapinfo.bSwapPageNo
#define bOvl		u.s1.swapinfo.bOvl
} UPACKED intr_pkt_t;
#define INTR_PKT_SIZE 28
