/*	$Id: ueaglevar.h,v 1.64 2004/08/29 19:07:57 damien Exp $	*/

/*-
 * Copyright (c) 2003, 2004
 *	Damien Bergamini <damien.bergamini@free.fr>. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice unmodified, this list of conditions, and the following
 *    disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/* macros and structure definitions for portability between *BSD */
#if defined(__NetBSD__) || defined(__OpenBSD__)
#define MS_TO_TICKS(ms) mstohz(ms)
#elif defined(__FreeBSD__)
#define MS_TO_TICKS(ms) ((ms) * hz / 1000)
#endif

#if defined(__FreeBSD__) && __FreeBSD_version >= 501000
#define GET_IFP(sc)	(&(sc)->ifatm.ifnet)
#else
#define GET_IFP(sc)	(&(sc)->ifnet)
#endif

#if defined(__NetBSD__) || defined(__OpenBSD__)
#define BPF_MTAP(ifp, m) do {						\
	if ((ifp)->if_bpf)						\
		bpf_mtap((ifp)->if_bpf, (m));				\
} while (/* CONSTCOND */0)
#endif


#define UEAGLE_NISOREQS		6
#define UEAGLE_NISOFRMS		4

#ifndef UEAGLE_INTR_INTERVAL
#define UEAGLE_INTR_INTERVAL	10	/* ms */
#endif

#define UEAGLE_TX_LIST_CNT	1

#define UEAGLE_MAXFWSIZE	(1024 * 1024)

#define UEAGLE_IDMA_TIMEOUT	10000
#define UEAGLE_TX_TIMEOUT	10000

#define UEAGLE_IFMTU		1500
#define UEAGLE_TXBUFLEN							\
	(((UEAGLE_IFMTU / ATM_CELL_PAYLOAD_SIZE) + 2) * ATM_CELL_SIZE)

#define CRC_INITIAL	0xffffffff
#define CRC_MAGIC	0xc704dd7b
#define CRC_POLYNOM	0x04c11db7

#define ATM_CELL_SIZE		53
#define ATM_CELL_HEADER_SIZE	5
#define ATM_CELL_PAYLOAD_SIZE	(ATM_CELL_SIZE - ATM_CELL_HEADER_SIZE)
#define AAL5_TRAILER_SIZE	8

/*-
 *    0                 4                 8
 *    +-----------------+-----------------+
 *    |       GFC	|	VPI	  |
 *    +-----------------+-----------------+
 *    |       VPI	|	VCI	  |
 *    +-----------------+-----------------+
 *    | 	       VCI		  |
 *    +-----------------+-----------+-----+
 *    |       VCI	|   PT (3)  | CLP |
 *    +-----------------+-----------+-----+
 *    | 	       HEC		  |
 *    +-----------------------------------+
 */
#define ATM_CH_FILL(x, gfc, vpi, vci, pt, clp, hec) do {		\
	(x)[0] = ((gfc) & 0xf) << 4 | ((vpi) & 0xf0) >> 4;		\
	(x)[1] = ((vpi) & 0xf) << 4 | ((vci) & 0xf000) >> 12;		\
	(x)[2] = ((vci) & 0xff0) >> 4;  				\
	(x)[3] = ((vci) & 0xf) << 4 | ((pt) & 0x7) << 1 | ((clp) & 0x1);\
	(x)[4] = (u_int8_t)(hec);					\
} while (/* CONSTCOND */0)

#define ATM_CH_SETPTFLAGS(x, v) ((x)[3] |= ((v) & 0x7) << 1)
#define ATM_CH_GETPTFLAGS(x) (((x)[3] >> 1) & 0x7)
#define ATM_CH_GETVPI(x) ((x)[0] << 4 | (x)[1] >> 4)
#define ATM_CH_GETVCI(x) \
	(((x)[1] & 0xf) << 12 | (x)[2] << 4 | ((x)[3] & 0xf0) >> 4)

/* optimized shortcut for (ATM_CH_GETPTFLAGS(x) & 1) */
#define ATM_CH_ISLASTCELL(x) ((x)[3] & 0x2)

#define AAL5_TR_SETCPSUU(x, v)	((x)[45] = (u_int8_t)(v))
#define AAL5_TR_SETCPI(x, v)	((x)[46] = (u_int8_t)(v))
#define AAL5_TR_SETPDULEN(x, v) do {					\
	(x)[47] = (u_int8_t)((v) >> 8);					\
	(x)[48] = (u_int8_t)(v);					\
} while (/* CONSTCOND */0)

#define AAL5_TR_GETPDULEN(x) (u_int16_t)((x)[47] << 8 | (x)[48])
#define AAL5_TR_SETCRC(x, v) do {					\
	(x)[49] = (u_int8_t)((v) >> 24);				\
	(x)[50] = (u_int8_t)((v) >> 16);				\
	(x)[51] = (u_int8_t)((v) >> 8); 				\
	(x)[52] = (u_int8_t)(v);					\
} while (/* CONSTCOND */0)

struct ueagle_vcc {
#if defined(__FreeBSD__) && __FreeBSD_version >= 502000
	struct atmio_vcc	param;
#else
	struct {
		u_int16_t	flags;
		u_int8_t	vpi;
		u_int16_t	vci;
	} param;
#endif
	void 			*rxhand;
	struct atm_pseudohdr	aph;
	struct mbuf		*m_head;
	u_int8_t		*dst;
	u_int8_t		*limit;
	u_int8_t		ch[ATM_CELL_HEADER_SIZE];
	int			drop;
};
#define UEAGLE_MAXVCC	1

struct ueagle_softc;

#define CMV_NORMAL_COMPLETION	0
#define CMV_TIMEOUT		1
#define CMV_ERROR		2
typedef void (*cmv_cb_f)(struct ueagle_softc *, int, u_int32_t);

struct ueagle_softc {
#if defined(__FreeBSD__)
	struct ifatm		ifatm;
	USBBASEDEVICE		sc_dev;

	struct mtx		mtx;
#else
	USBBASEDEVICE		sc_dev;
	struct ifnet		ifnet;

	struct usb_task         sc_task;
	u_int16_t		pageno;
	u_int16_t		ovl;
#endif
	usbd_device_handle	sc_udev;

	usbd_pipe_handle	ep_tx;
	usbd_pipe_handle	ep_rx;
	usbd_pipe_handle	ep_idma;
	usbd_pipe_handle	ep_intr;

	int			gone;

	struct isoreq {
		struct ueagle_softc	*sc;
		usbd_xfer_handle	xfer;
		u_int16_t		frlengths[UEAGLE_NISOFRMS];
		u_char			*offsets[UEAGLE_NISOFRMS];
	} isoreqs[UEAGLE_NISOREQS];

        struct txreq {
                struct ueagle_softc     *sc;
                usbd_xfer_handle        xfer;
                u_char                  *buf;
        } txreqs[UEAGLE_TX_LIST_CNT];

	u_int8_t		stage;

	usb_callout_t		sc_ch;

	void			*ibuf;
	u_int			isize;

	u_int16_t		cmv_idx;
	u_int16_t		cmv_expected;
	u_int32_t 		cmv_address;
	u_int16_t		cmv_offset;
	cmv_cb_f		cmv_cb;

	struct ueagle_vcc	*vccs[UEAGLE_MAXVCC];
	int			open_vccs;

	/* keep in sync with eaglectl */
	struct ueagle_stats {
		struct {
			u_int32_t	state;
			u_int32_t	flags;
			u_int32_t	vidcpe;
			u_int32_t	vidco;
			u_int32_t	dsrate;
			u_int32_t	usrate;
			u_int32_t	dserror;
			u_int32_t	userror;
			u_int32_t	dsunc;
			u_int32_t	usunc;
			u_int32_t	txflow;
			u_int32_t	rxflow;
			u_int32_t	attenuation;
			u_int32_t	dsmargin;
			u_int32_t	usmargin;
			u_int32_t	cells_crc_errors;
		} phy;

		struct {
			u_int32_t	cells_transmitted;
			u_int32_t	cells_received;
			u_int32_t	cells_dropped;
			u_int32_t	vcc_no_conn;
			u_int32_t	cspdus_transmitted;
			u_int32_t	cspdus_received;
			u_int32_t	cspdus_crc_errors;
			u_int32_t	cspdus_dropped;
		} atm;
	} stats;
};

/* constants and structures for ioctls (must be kept in sync with eaglectl) */
#define SIOCGATMSTATS		_IOWR('i', 138, struct ifreq)
#define SIOCGUSBDEVICE		_IOWR('i', 139, struct ifreq)
#define SIOCGUSBCONFIG		_IOWR('i', 140, struct ifreq)
#define SIOCGUSBFULLDESC	_IOWR('i', 141, struct ifreq)

struct ueagle_atmvcc {
	u_int8_t	flags;
	u_int8_t	vpi;
	u_int16_t	vci;
};
